#!/usr/bin/env python
# encoding: utf-8

"""Extension providing convenience functions for working with subflows.

Embedded flow are very convenient to avoid job duplication (e.g. when running
different variants of a workflow with different set of parameters). The classes
below provide a concise way to leverage flow job options (propagated to all
jobs inside the target subflow) to generate dynamic options.

"""

from ..job import Job
from ..util import flatten


class FlowProperty(object):

  """Dynamic property.

  These job properties are dynamically populated depending on which subflow
  the job is embedded in.

  Note that this class (similarly to the `Project` class) is not thread-safe
  and should only be instantiated from a single thread.

  """

  __register = {}

  def __init__(self, **kwargs):
    self.key = 'flow.property.%s' % (len(self.__register), )
    self.__register[self.key] = kwargs

  def __str__(self):
    return '${%s}' % (self.key, )

  @classmethod
  def get_options(cls, mode):
    """Retrieve all properties for a given mode.

    :param mode: Which dynamic properties to select.

    This should only be called once all dynamic properties have been added (at
    least for the input mode).

    """
    return dict(
      (key, properties[mode])
      for key, properties in cls.__register.items()
      if mode in properties
    )

  @classmethod
  def reset(cls):
    """Reset all properties.

    Mostly only used by tests.

    """
    cls.__register = {}


class FlowJob(Job):

  """Job class for embedded flows.

  :param subflow: Subflow name. Will take precedence over any `'flow.name'`
    option specified in `options`.
  :param mode: Used for dynamic `FlowProperty`'s.
  :param \*options: Regular `Job` options.

  """

  def __init__(self, subflow, mode, *options):
    super(FlowJob, self).__init__(*options)
    self.mode = mode
    self.options['type'] = 'flow'
    self.options['flow.name'] = subflow

  def build(self, *args, **kwargs):
    """Build job options.

    :param \*args: Positional arguments passed to the base `build` function.
    :param \*\*kwargs: Keyword arguments passed to the base `build` function.

    We delay the full expansion of options until now to make sure that all
    dynamic properties have already been added. Note also that each `FlowJob`
    will contain the list of all dynamic properties defined for its mode.

    """
    self.options.update(FlowProperty.get_options(self.mode))
    super(FlowJob, self).build(*args, **kwargs)



#!/usr/bin/env python
# encoding: utf-8

"""Test Azkaban pig extension.

Note the type `'pig'` enforcement to override a potential configuration option.

"""

from azkaban.ext.flow import *
from azkaban.util import read_properties, temppath
from nose.tools import eq_


class TestFlowProperty(object):

  def setup(self):
    FlowProperty.reset()

  def test_init(self):
    p1 = FlowProperty(mode1='a', mode2='b')
    p2 = FlowProperty(mode1='A', mode3='C')
    eq_(p1.key, 'flow.property.0')
    eq_(p2.key, 'flow.property.1')

  def test_str(self):
    job = Job({'foo': {'bar': FlowProperty(mode1='a', mode2='b')}})
    with temppath() as tpath:
      job.build(path=tpath)
      eq_(read_properties(tpath), {'foo.bar': '${flow.property.0}'})

  def test_get_options(self):
    p1 = FlowProperty(mode1='a', mode2='b')
    p2 = FlowProperty(mode1='A', mode3='C')
    eq_(FlowProperty.get_options('mode1'), {p1.key: 'a', p2.key: 'A'})
    eq_(FlowProperty.get_options('mode3'), {p2.key: 'C'})
    eq_(FlowProperty.get_options('mode4'), {})


class TestFlowJob(object):

  def setup(self):
    FlowProperty.reset()

  def test_options(self):
    job = FlowJob('foo', 'mode1', {'type': 'noop', 'bar': 2, 'flow.name': ''})
    eq_(job.options, {'type': 'flow', 'flow.name': 'foo', 'bar': 2})

  def test_build(self):
    p1 = FlowProperty(mode1='a', mode2='b')
    p2 = FlowProperty(mode1='A', mode3='C')
    job = FlowJob('foo', 'mode1')
    with temppath() as tpath:
      job.build(path=tpath)
      eq_(read_properties(tpath), {
        'type': 'flow',
        'flow.name': 'foo',
        'flow.property.0': 'a',
        'flow.property.1': 'A',
      })
